import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.util.List;

/**
 * A ball is an object that can hit other objects and bounce off the 
 * edges of the world (except the bottom edge).  It will bounce off of
 * a paddle as well.
 * 
 * @author Barbara Ericson Georgia Tech 
 * @version 1.0 April 6, 2007
 */
public class Ball extends Actor
{

    //////////// fields /////////////////////////////
    
    /** the radius of this ball */
    private int radius = 10;
    
    /** the width of this ball (diameter) */
    private int width = radius * 2;
    
    /** the color of the ball */
    private Color color = Color.BLACK;
    
    /** the amount of change in x during each act */
    private int velX;
    
    /** the amount of change in y during each act */
    private int velY = 3;
    
    /** fireball effect */
    private boolean fire = false;
    
    ////////////////// constructors /////////////////////
    
    /**
     * Constructor that takes no arguments 
     */
    public Ball()
    {
        velX = Greenfoot.getRandomNumber(3) + 1;
        if (Greenfoot.getRandomNumber(2) == 0)
           velX = -1 * velX;
        updateImage();
    }
    
    /**
     * Constructor that takes initial values for all fields
     * @param theRadius the radius to use
     * @param theColor  the color to use
     * @param theVelX   the amount to change in X per act
     * @param theVelY   the amount to change in Y per act
     */
    public Ball(int theRadius, Color theColor, 
                int theVelX, int theVelY)
    {
        radius = theRadius;
        color = theColor;
        velX = theVelX;
        velY = theVelY;
        updateImage();
    }
    
    ///////////////////// Methods ///////////////////////////////
    
    /**
     * Balls will move and check if they have hit a brick or paddle or
     * one of the edges of the world
     */
    public void act() 
    {
        BreakoutWorld world = (BreakoutWorld) getWorld();
        // move the ball
        
        int bX = getX();
        int bY = getY();
        setLocation(bX+velX,bY+velY);
        
        
        bX = getX();
        bY = getY();
        // see if hit any actor
        List as = getIntersectingObjects(null);
        for (int i = 0; i<as.size(); i++)
        {
            Actor a = (Actor) as.get(i);
            if (a instanceof Message || a instanceof Powers)
            {
                as.remove(i);
                i--;
            }
        }
        
        Actor a = null;
        if (as.size()>0)
            a = (Actor) as.get(0);
    
        // if hit an actor and not a message object 
        if (a != null && !(a instanceof Message) && !(a instanceof Powers))
        {
            
            /* if it was a brick then remove it and check if won */
            if (a instanceof Brick)
            {
                int pX = ((Brick)a).getX();
                int pY = ((Brick)a).getY();
                if (!fire)
                    velY*=-1;
                world.removeObject(a);
                Greenfoot.playSound("brick.WAV");
                if (Greenfoot.getRandomNumber(9) == 0)
                    new Powers(world, pX, pY);
                world.checkIfWon();
            }
            else
            {
                int pX = ((Paddle)a).getX();
                int pY = ((Paddle)a).getY();
                double dX = (double)bX-(double)pX;
                double offset = dX/Math.abs(dX)*.32;
                /**   reflect ball in direction based on where it hit the padle   */
                changeSpeed( (int)((3*(offset+(dX/(((double)a.getWidth())/2))) )) ,-velY);
                Greenfoot.playSound("padle.WAV");
            }
        }        
        // check if hit walls (left wall first)
        if (bX<=radius)
        {
            velX=-velX;
            Greenfoot.playSound("wall.WAV");
        }
        // right wall
        if (bX>=world.getWidth()-radius)
        {
            velX=-velX;
            Greenfoot.playSound("wall.WAV");
        }
        // top wall
        if (bY<=radius)
        {
            velY=-velY;
            Greenfoot.playSound("wall.WAV");
        }
        // bottom wall
        if (bY>=world.getHeight()-radius)
        {
            // revmoe the ball from the world
            world.removeObject(this);
            
            // create a new ball
            world.newBall();
        }
          
    }
    
    /**
     * Method to set the ball color
     */
    public void setColor(Color theColor)
    {
        this.color = theColor;
        updateImage();
    }
    
    /**
     * Method to create the image and set it for the ball 
     * If you change the ball width or color you should 
     * invoke this again
     */
    public void updateImage()
    {
        GreenfootImage image = new GreenfootImage(width,width);
        image.setColor(color);
        image.fillOval(0,0,width,width);
        setImage(image);
    }
    public void turnFire(boolean f)
    {
        fire = f;
        if (f)
            setColor(Color.RED);
        else
            setColor(Color.BLACK);
    }
    public int getVX()
    {
        return velX;
    }
    public int getVY()
    {
        return velY;
    }
    public void changeSpeed(int vX, int vY)
    {
        if (vY ==0)
            vY=1;
        velX=vX;
        velY=vY;        
    }
}
